/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.dubbo.rpc.protocol.tri.command;

import org.apache.dubbo.rpc.protocol.tri.stream.TripleStreamChannelFuture;

import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPromise;
import io.netty.handler.codec.http2.Http2StreamChannel;
import io.netty.handler.codec.http2.Http2StreamChannelBootstrap;
import io.netty.util.concurrent.Future;

public class CreateStreamQueueCommand extends QueuedCommand {

    private final Http2StreamChannelBootstrap bootstrap;

    private final TripleStreamChannelFuture streamChannelFuture;

    private CreateStreamQueueCommand(
            Http2StreamChannelBootstrap bootstrap, TripleStreamChannelFuture streamChannelFuture) {
        this.bootstrap = bootstrap;
        this.streamChannelFuture = streamChannelFuture;
        this.promise(streamChannelFuture.getParentChannel().newPromise());
        this.channel(streamChannelFuture.getParentChannel());
    }

    public static CreateStreamQueueCommand create(
            Http2StreamChannelBootstrap bootstrap, TripleStreamChannelFuture streamChannelFuture) {
        return new CreateStreamQueueCommand(bootstrap, streamChannelFuture);
    }

    @Override
    public void doSend(ChannelHandlerContext ctx, ChannelPromise promise) {
        // NOOP
    }

    @Override
    public void run(Channel channel) {
        // work in I/O thread
        Future<Http2StreamChannel> future = bootstrap.open();
        if (future.isSuccess()) {
            streamChannelFuture.complete(future.getNow());
        } else {
            streamChannelFuture.completeExceptionally(future.cause());
        }
    }
}
